@@ -1,4 +1,4 @@
-/* $OpenBSD: tasn_prn.c,v 1.21 2020/03/24 10:46:38 inoguchi Exp $ */
+/* $OpenBSD: tasn_prn.c,v 1.25 2023/07/05 21:23:36 beck Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 2000.
  */
@@ -65,104 +65,107 @@
 #include <openssl/objects.h>
 #include <openssl/x509v3.h>
 
-#include "asn1_locl.h"
+#include "asn1_local.h"
 
 /* Print routines.
  */
 
 /* ASN1_PCTX routines */
 
-ASN1_PCTX default_pctx = {
-	ASN1_PCTX_FLAGS_SHOW_ABSENT,	/* flags */
-	0,				/* nm_flags */
-	0,				/* cert_flags */
-	0,				/* oid_flags */
-	0				/* str_flags */
+static const ASN1_PCTX default_pctx = {
+	.flags = ASN1_PCTX_FLAGS_SHOW_ABSENT,
 };
 
-
 ASN1_PCTX *
 ASN1_PCTX_new(void)
 {
-	ASN1_PCTX *ret;
-	ret = malloc(sizeof(ASN1_PCTX));
-	if (ret == NULL) {
+	ASN1_PCTX *p;
+
+	if ((p = calloc(1, sizeof(ASN1_PCTX))) == NULL) {
 		ASN1error(ERR_R_MALLOC_FAILURE);
 		return NULL;
 	}
-	ret->flags = 0;
-	ret->nm_flags = 0;
-	ret->cert_flags = 0;
-	ret->oid_flags = 0;
-	ret->str_flags = 0;
-	return ret;
+
+	return p;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_new);
 
 void
 ASN1_PCTX_free(ASN1_PCTX *p)
 {
 	free(p);
 }
+LCRYPTO_ALIAS(ASN1_PCTX_free);
 
 unsigned long
 ASN1_PCTX_get_flags(const ASN1_PCTX *p)
 {
 	return p->flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_get_flags);
 
 void
 ASN1_PCTX_set_flags(ASN1_PCTX *p, unsigned long flags)
 {
 	p->flags = flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_set_flags);
 
 unsigned long
 ASN1_PCTX_get_nm_flags(const ASN1_PCTX *p)
 {
 	return p->nm_flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_get_nm_flags);
 
 void
 ASN1_PCTX_set_nm_flags(ASN1_PCTX *p, unsigned long flags)
 {
 	p->nm_flags = flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_set_nm_flags);
 
 unsigned long
 ASN1_PCTX_get_cert_flags(const ASN1_PCTX *p)
 {
 	return p->cert_flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_get_cert_flags);
 
 void
 ASN1_PCTX_set_cert_flags(ASN1_PCTX *p, unsigned long flags)
 {
 	p->cert_flags = flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_set_cert_flags);
 
 unsigned long
 ASN1_PCTX_get_oid_flags(const ASN1_PCTX *p)
 {
 	return p->oid_flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_get_oid_flags);
 
 void
 ASN1_PCTX_set_oid_flags(ASN1_PCTX *p, unsigned long flags)
 {
 	p->oid_flags = flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_set_oid_flags);
 
 unsigned long
 ASN1_PCTX_get_str_flags(const ASN1_PCTX *p)
 {
 	return p->str_flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_get_str_flags);
 
 void
 ASN1_PCTX_set_str_flags(ASN1_PCTX *p, unsigned long flags)
 {
 	p->str_flags = flags;
 }
+LCRYPTO_ALIAS(ASN1_PCTX_set_str_flags);
 
 /* Main print routines */
 
@@ -195,6 +198,7 @@ ASN1_item_print(BIO *out, ASN1_VALUE *ifld, int indent, const ASN1_ITEM *it,
 	return asn1_item_print_ctx(out, &ifld, indent, it, NULL, sname,
 	    0, pctx);
 }
+LCRYPTO_ALIAS(ASN1_item_print);
 
 static int
 asn1_item_print_ctx(BIO *out, ASN1_VALUE **fld, int indent, const ASN1_ITEM *it,
