@@ -14,9 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-#if defined(_MSC_VER)
-	#pragma warning ( disable: 4231 4251 4275 4786 )
-#endif
 
 #include <log4cxx/logstring.h>
 #include <log4cxx/patternlayout.h>
@@ -28,6 +25,9 @@
 #include <log4cxx/helpers/optionconverter.h>
 
 #include <log4cxx/pattern/loggerpatternconverter.h>
+#include <log4cxx/pattern/colorendpatternconverter.h>
+#include <log4cxx/pattern/colorstartpatternconverter.h>
+#include <log4cxx/pattern/shortfilelocationpatternconverter.h>
 #include <log4cxx/pattern/literalpatternconverter.h>
 #include <log4cxx/helpers/loglog.h>
 #include <log4cxx/pattern/classnamepatternconverter.h>
@@ -43,9 +43,11 @@
 #include <log4cxx/pattern/levelpatternconverter.h>
 #include <log4cxx/pattern/relativetimepatternconverter.h>
 #include <log4cxx/pattern/threadpatternconverter.h>
+#include <log4cxx/pattern/mdcpatternconverter.h>
 #include <log4cxx/pattern/ndcpatternconverter.h>
 #include <log4cxx/pattern/propertiespatternconverter.h>
 #include <log4cxx/pattern/throwableinformationpatternconverter.h>
+#include <log4cxx/pattern/threadusernamepatternconverter.h>
 
 
 using namespace log4cxx;
@@ -53,26 +55,58 @@ using namespace log4cxx::helpers;
 using namespace log4cxx::spi;
 using namespace log4cxx::pattern;
 
+struct PatternLayout::PatternLayoutPrivate
+{
+	PatternLayoutPrivate() {}
+	PatternLayoutPrivate(const LogString& pattern) :
+		conversionPattern(pattern) {}
+
+	/**
+	 * Conversion pattern.
+	 */
+	LogString conversionPattern;
+
+	/**
+	 * Pattern converters.
+	 */
+	LoggingEventPatternConverterList patternConverters;
+
+	/**
+	 * Field widths and alignment corresponding to pattern converters.
+	 */
+	FormattingInfoList patternFields;
+
+	LogString m_fatalColor = LOG4CXX_STR("\\x1B[35m"); //magenta
+	LogString m_errorColor = LOG4CXX_STR("\\x1B[31m"); //red
+	LogString m_warnColor = LOG4CXX_STR("\\x1B[33m"); //yellow
+	LogString m_infoColor = LOG4CXX_STR("\\x1B[32m"); //green
+	LogString m_debugColor = LOG4CXX_STR("\\x1B[36m"); //cyan;
+	LogString m_traceColor = LOG4CXX_STR("\\x1B[34m"); //blue;
+};
+
 IMPLEMENT_LOG4CXX_OBJECT(PatternLayout)
 
 
-PatternLayout::PatternLayout()
+PatternLayout::PatternLayout() :
+	m_priv(std::make_unique<PatternLayoutPrivate>())
 {
 }
 
 /**
 Constructs a PatternLayout using the supplied conversion pattern.
 */
-PatternLayout::PatternLayout(const LogString& pattern)
-	: conversionPattern(pattern)
+PatternLayout::PatternLayout(const LogString& pattern) :
+	m_priv(std::make_unique<PatternLayoutPrivate>(pattern))
 {
 	Pool pool;
 	activateOptions(pool);
 }
 
+PatternLayout::~PatternLayout() {}
+
 void PatternLayout::setConversionPattern(const LogString& pattern)
 {
-	conversionPattern = pattern;
+	m_priv->conversionPattern = pattern;
 	Pool pool;
 	activateOptions(pool);
 }
@@ -82,11 +116,11 @@ void PatternLayout::format(LogString& output,
 	Pool& pool) const
 {
 	std::vector<FormattingInfoPtr>::const_iterator formatterIter =
-		patternFields.begin();
+		m_priv->patternFields.begin();
 
 	for (std::vector<LoggingEventPatternConverterPtr>::const_iterator
-		converterIter = patternConverters.begin();
-		converterIter != patternConverters.end();
+		converterIter = m_priv->patternConverters.begin();
+		converterIter != m_priv->patternConverters.end();
 		converterIter++, formatterIter++)
 	{
 		int startField = (int)output.length();
@@ -102,25 +136,51 @@ void PatternLayout::setOption(const LogString& option, const LogString& value)
 			LOG4CXX_STR("CONVERSIONPATTERN"),
 			LOG4CXX_STR("conversionpattern")))
 	{
-		conversionPattern = OptionConverter::convertSpecialChars(value);
+		m_priv->conversionPattern = OptionConverter::convertSpecialChars(value);
+	}else if(StringHelper::equalsIgnoreCase(option,
+											LOG4CXX_STR("ERRORCOLOR"),
+											LOG4CXX_STR("errorcolor"))){
+		m_priv->m_errorColor = value;
+		LogLog::debug(LOG4CXX_STR("Setting error color to "));
+		LogLog::debug(value);
+	}else if(StringHelper::equalsIgnoreCase(option,
+											LOG4CXX_STR("FATALCOLOR"),
+											LOG4CXX_STR("fatalcolor"))){
+		m_priv->m_fatalColor = value;
+	}else if(StringHelper::equalsIgnoreCase(option,
+											LOG4CXX_STR("WARNCOLOR"),
+											LOG4CXX_STR("warncolor"))){
+		m_priv->m_warnColor = value;
+	}else if(StringHelper::equalsIgnoreCase(option,
+											LOG4CXX_STR("INFOCOLOR"),
+											LOG4CXX_STR("infocolor"))){
+		m_priv->m_infoColor = value;
+	}else if(StringHelper::equalsIgnoreCase(option,
+											LOG4CXX_STR("DEBUGCOLOR"),
+											LOG4CXX_STR("debugcolor"))){
+		m_priv->m_debugColor = value;
+	}else if(StringHelper::equalsIgnoreCase(option,
+											LOG4CXX_STR("TRACECOLOR"),
+											LOG4CXX_STR("tracecolor"))){
+		m_priv->m_traceColor = value;
 	}
 }
 
 void PatternLayout::activateOptions(Pool&)
 {
-	LogString pat(conversionPattern);
+	LogString pat(m_priv->conversionPattern);
 
 	if (pat.empty())
 	{
 		pat = LOG4CXX_STR("%m%n");
 	}
 
-	patternConverters.erase(patternConverters.begin(), patternConverters.end());
-	patternFields.erase(patternFields.begin(), patternFields.end());
+	m_priv->patternConverters.erase(m_priv->patternConverters.begin(), m_priv->patternConverters.end());
+	m_priv->patternFields.erase(m_priv->patternFields.begin(), m_priv->patternFields.end());
 	std::vector<PatternConverterPtr> converters;
 	PatternParser::parse(pat,
 		converters,
-		patternFields,
+		m_priv->patternFields,
 		getFormatSpecifiers());
 
 	//
@@ -131,17 +191,18 @@ void PatternLayout::activateOptions(Pool&)
 		converterIter != converters.end();
 		converterIter++)
 	{
-		LoggingEventPatternConverterPtr eventConverter(*converterIter);
+		LoggingEventPatternConverterPtr eventConverter =
+			log4cxx::cast<LoggingEventPatternConverter>(*converterIter);
 
 		if (eventConverter != NULL)
 		{
-			patternConverters.push_back(eventConverter);
+			m_priv->patternConverters.push_back(eventConverter);
 		}
 	}
 }
 
 #define RULES_PUT(spec, cls) \
-	specs.insert(PatternMap::value_type(LogString(LOG4CXX_STR(spec)), (PatternConstructor) cls ::newInstance))
+	specs.insert(PatternMap::value_type(LogString(LOG4CXX_STR(spec)), cls ::newInstance))
 
 
 log4cxx::pattern::PatternMap PatternLayout::getFormatSpecifiers()
@@ -153,9 +214,14 @@ log4cxx::pattern::PatternMap PatternLayout::getFormatSpecifiers()
 	RULES_PUT("C", ClassNamePatternConverter);
 	RULES_PUT("class", ClassNamePatternConverter);
 
+	specs.insert(PatternMap::value_type(LogString(LOG4CXX_STR("Y")), std::bind(&PatternLayout::createColorStartPatternConverter, this, std::placeholders::_1)));
+	RULES_PUT("y", ColorEndPatternConverter);
+
 	RULES_PUT("d", DatePatternConverter);
 	RULES_PUT("date", DatePatternConverter);
 
+	RULES_PUT("f", ShortFileLocationPatternConverter);
+
 	RULES_PUT("F", FileLocationPatternConverter);
 	RULES_PUT("file", FileLocationPatternConverter);
 
@@ -181,17 +247,37 @@ log4cxx::pattern::PatternMap PatternLayout::getFormatSpecifiers()
 	RULES_PUT("t", ThreadPatternConverter);
 	RULES_PUT("thread", ThreadPatternConverter);
 
+	RULES_PUT("T", ThreadUsernamePatternConverter);
+	RULES_PUT("threadname", ThreadUsernamePatternConverter);
+
 	RULES_PUT("x", NDCPatternConverter);
 	RULES_PUT("ndc", NDCPatternConverter);
 
 	RULES_PUT("X", PropertiesPatternConverter);
+	RULES_PUT("J", MDCPatternConverter);
 	RULES_PUT("properties", PropertiesPatternConverter);
 
 	RULES_PUT("throwable", ThrowableInformationPatternConverter);
 	return specs;
 }
 
+LogString PatternLayout::getConversionPattern() const
+{
+	return m_priv->conversionPattern;
+}
 
+pattern::PatternConverterPtr PatternLayout::createColorStartPatternConverter(const std::vector<LogString>& options){
+	std::shared_ptr<ColorStartPatternConverter> colorPatternConverter = std::make_shared<ColorStartPatternConverter>();
+
+	colorPatternConverter->setErrorColor(m_priv->m_errorColor);
+	colorPatternConverter->setFatalColor(m_priv->m_fatalColor);
+	colorPatternConverter->setWarnColor(m_priv->m_warnColor);
+	colorPatternConverter->setInfoColor(m_priv->m_infoColor);
+	colorPatternConverter->setDebugColor(m_priv->m_debugColor);
+	colorPatternConverter->setTraceColor(m_priv->m_traceColor);
+
+	return colorPatternConverter;
+}
 
 
 
