@@ -18,12 +18,6 @@
 #if !defined(_LOG4CXX_ROLLING_ROLLING_POLICY_BASE_H)
 #define _LOG4CXX_ROLLING_ROLLING_POLICY_BASE_H
 
-#if defined(_MSC_VER)
-	#pragma warning ( push )
-	#pragma warning ( disable: 4231 4251 4275 4786 )
-#endif
-
-
 #include <log4cxx/helpers/object.h>
 #include <log4cxx/logger.h>
 #include <log4cxx/logmanager.h>
@@ -48,7 +42,7 @@ LOG4CXX_LIST_DEF(FormattingInfoList, log4cxx::pattern::FormattingInfoPtr);
  */
 class LOG4CXX_EXPORT RollingPolicyBase :
 	public virtual RollingPolicy,
-	public virtual helpers::ObjectImpl
+	public virtual helpers::Object
 {
 	protected:
 		DECLARE_ABSTRACT_LOG4CXX_OBJECT(RollingPolicyBase)
@@ -57,34 +51,15 @@ class LOG4CXX_EXPORT RollingPolicyBase :
 		LOG4CXX_CAST_ENTRY(spi::OptionHandler)
 		END_LOG4CXX_CAST_MAP()
 
-
-	private:
-		/**
-		 * File name pattern converters.
-		 */
-		PatternConverterList patternConverters;
-
-		/**
-		 * File name field specifiers.
-		 */
-		FormattingInfoList patternFields;
-
-		/**
-		 * File name pattern.
-		 */
-		LogString fileNamePatternStr;
-
+		LOG4CXX_DECLARE_PRIVATE_MEMBER_PTR(RollingPolicyBasePrivate, m_priv)
 
 	public:
 		RollingPolicyBase();
 		virtual ~RollingPolicyBase();
-		void addRef() const;
-		void releaseRef() const;
-		virtual void activateOptions(log4cxx::helpers::Pool& p) = 0;
+		void activateOptions(log4cxx::helpers::Pool& p) override;
 		virtual log4cxx::pattern::PatternMap getFormatSpecifiers() const = 0;
 
-		virtual void setOption(const LogString& option,
-			const LogString& value);
+		void setOption(const LogString& option, const LogString& value) override;
 
 		/**
 		 * Set file name pattern.
@@ -98,14 +73,13 @@ class LOG4CXX_EXPORT RollingPolicyBase :
 		 */
 		LogString getFileNamePattern() const;
 
+		bool getCreateIntermediateDirectories() const;
+		void setCreateIntermediateDirectories(bool createIntermediate);
+
+		PatternConverterList getPatternConverterList() const;
 
-#ifdef LOG4CXX_MULTI_PROCESS
-		PatternConverterList getPatternConverterList()
-		{
-			return patternConverters;
-		}
-#endif
 	protected:
+		RollingPolicyBase(LOG4CXX_PRIVATE_PTR(RollingPolicyBasePrivate) priv);
 		/**
 		 *   Parse file name pattern.
 		 */
@@ -118,20 +92,16 @@ class LOG4CXX_EXPORT RollingPolicyBase :
 		 * @param buf string buffer to which formatted file name is appended, may not be null.
 		 * @param p memory pool.
 		 */
-		void formatFileName(log4cxx::helpers::ObjectPtr& obj,
-			LogString& buf, log4cxx::helpers::Pool& p) const;
+		void formatFileName(const helpers::ObjectPtr& obj,
+			LogString& buf, helpers::Pool& p) const;
 
 		log4cxx::pattern::PatternConverterPtr getIntegerPatternConverter() const;
 		log4cxx::pattern::PatternConverterPtr getDatePatternConverter() const;
-
-
 };
+
+LOG4CXX_PTR_DEF(RollingPolicyBase);
+
 }
 }
 
-
-#if defined(_MSC_VER)
-	#pragma warning ( pop )
-#endif
-
 #endif
