@@ -48,6 +48,7 @@
 // Copyright (C) 2022 Claes Nästén <pekdon@gmail.com>
 // Copyright (C) 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright (C) 2023 Shivodit Gill <shivodit.gill@gmail.com>
+// Copyright (C) 2024 Keyu Tao <me@taoky.moe>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -1163,11 +1164,16 @@ UCharFontSearchResult GlobalParams::findSystemFontFileForUChar(Unicode uChar, co
             int faceIndex = 0;
             FcChar8 *fcFamily = nullptr;
             FcChar8 *fcStyle = nullptr;
+            FcCharSet *fcCharSet = nullptr;
             FcPatternGetString(fontSet->fonts[i], FC_FILE, 0, &fcFilePath);
             FcPatternGetInteger(fontSet->fonts[i], FC_INDEX, 0, &faceIndex);
             FcPatternGetString(fontSet->fonts[i], FC_FAMILY, 0, &fcFamily);
             FcPatternGetString(fontSet->fonts[i], FC_STYLE, 0, &fcStyle);
-            if (!fcFilePath || !fcFamily || !fcStyle) {
+            FcPatternGetCharSet(fontSet->fonts[i], FC_CHARSET, 0, &fcCharSet);
+            if (!fcFilePath || !fcFamily || !fcStyle || !fcCharSet) {
+                continue;
+            }
+            if (!FcCharSetHasChar(fcCharSet, uChar)) {
                 continue;
             }
 
@@ -1560,7 +1566,9 @@ void GlobalParams::setErrQuiet(bool errQuietA)
 #ifdef ANDROID
 void GlobalParams::setFontDir(const std::string &fontDir)
 {
+#    if defined(WITH_FONTCONFIGURATION_ANDROID)
     displayFontDir = fontDir;
+#    endif
 }
 #endif
 
