@@ -3,7 +3,7 @@
  * @author Christian Schenk
  * @brief PackageManager implementation
  *
- * @copyright Copyright © 2001-2024 Christian Schenk
+ * @copyright Copyright © 2001-2026 Christian Schenk
  *
  * This file is part of MiKTeX Package Manager.
  *
@@ -43,7 +43,7 @@ using namespace std;
 
 using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
-using namespace MiKTeX::Extractor;
+using namespace MiKTeX::Archive;
 using namespace MiKTeX::Packages;
 using namespace MiKTeX::Trace;
 using namespace MiKTeX::Util;
@@ -153,7 +153,7 @@ void PackageManagerImpl::LoadDatabase(const PathName& path, bool isArchive)
         tempDir = TemporaryDirectory::Create();
 
         // extract "package-manifests.ini" from archive
-        unique_ptr<MiKTeX::Extractor::Extractor> extractor(MiKTeX::Extractor::Extractor::CreateExtractor(DB_ARCHIVE_FILE_TYPE));
+        unique_ptr<MiKTeX::Archive::Extractor> extractor(MiKTeX::Archive::Extractor::New(DB_ARCHIVE_FILE_TYPE));
         extractor->Extract(absPath, tempDir->GetPathName());
 
         packageManifestsPath = tempDir->GetPathName() / MIKTEX_PACKAGE_MANIFESTS_INI_FILENAME;
@@ -1212,21 +1212,21 @@ InstallationSummary PackageManagerImpl::GetInstallationSummary(bool userScope)
         userScope ? MIKTEX_CONFIG_VALUE_LAST_USER_UPDATE_CHECK : MIKTEX_CONFIG_VALUE_LAST_ADMIN_UPDATE_CHECK,
         lastUpdateCheckText))
     {
-        result.lastUpdateCheck = std::stol(lastUpdateCheckText);
+        result.lastUpdateCheck = std::stoll(lastUpdateCheckText);
     }
     string lastUpdateText;
     if (session->TryGetConfigValue(MIKTEX_CONFIG_SECTION_MPM,
         userScope ? MIKTEX_CONFIG_VALUE_LAST_USER_UPDATE : MIKTEX_CONFIG_VALUE_LAST_ADMIN_UPDATE,
         lastUpdateText))
     {
-        result.lastUpdate = std::stol(lastUpdateText);
+        result.lastUpdate = std::stoll(lastUpdateText);
     }
     string lastUpdateDbText;
     if (session->TryGetConfigValue(MIKTEX_CONFIG_SECTION_MPM,
         userScope ? MIKTEX_CONFIG_VALUE_LAST_USER_UPDATE_DB : MIKTEX_CONFIG_VALUE_LAST_ADMIN_UPDATE_DB,
         lastUpdateDbText))
     {
-        result.lastUpdateDb = std::stol(lastUpdateDbText);
+        result.lastUpdateDb = std::stoll(lastUpdateDbText);
     }
     return result;
 }
