@@ -3,7 +3,7 @@
  * @author Christian Schenk
  * @brief Setup service implementation
  *
- * @copyright Copyright © 2013-2024 Christian Schenk
+ * @copyright Copyright © 2013-2026 Christian Schenk
  *
  * This file is part of the MiKTeX Setup Library.
  *
@@ -15,6 +15,8 @@
 
 #include "setup-version.h"
 
+#include <miktex/Archive/Creator>
+
 #include "internal.h"
 
 #if defined(MIKTEX_WINDOWS)
@@ -26,9 +28,10 @@ using namespace std::string_literals;
 
 using namespace nlohmann;
 
+using namespace MiKTeX::Archive;
 using namespace MiKTeX::Configuration;
 using namespace MiKTeX::Core;
-using namespace MiKTeX::Extractor;
+using namespace MiKTeX::Archive;
 using namespace MiKTeX::Locale;
 using namespace MiKTeX::Packages;
 using namespace MiKTeX::Setup;
@@ -351,7 +354,7 @@ unique_ptr<TemporaryDirectory> SetupService::ExtractFiles()
             memcmp(magic + 12, MAGIC3, 3) == 0 &&
             memcmp(magic + 15, MAGIC3, 1) == 0)
         {
-            unique_ptr<MiKTeX::Extractor::Extractor> extractor(MiKTeX::Extractor::Extractor::CreateExtractor(MiKTeX::Extractor::ArchiveFileType::Tar));
+            unique_ptr<MiKTeX::Archive::Extractor> extractor(MiKTeX::Archive::Extractor::New(MiKTeX::Archive::ArchiveFileType::Tar));
             unique_ptr<TemporaryDirectory> sfxDir = TemporaryDirectory::Create();
             extractor->Extract(&myImage, sfxDir->GetPathName(), true);
             return sfxDir;
@@ -2197,4 +2200,51 @@ vector<Issue> SetupServiceImpl::GetIssues()
 
     }
     return issues;
-}
\ No newline at end of file
+}
+
+void SetupServiceImpl::CollectDiagnosticInfo(const PathName& outputFileName)
+{
+    vector<FileSet> fileSets;
+    auto tmpDir = TemporaryDirectory::Create();
+    auto reportFileName = tmpDir->GetPathName() / "report.txt";
+    ofstream ofs;
+    ofs.open(reportFileName.ToString(), ios::out | ios::binary);
+    if (!ofs.is_open())
+    {
+        MIKTEX_FATAL_ERROR("The report could not be written.");
+    }
+    this->WriteReport(ofs, { ReportOption::General, ReportOption::RootDirectories, ReportOption::CurrentUser });
+    ofs.close();
+    fileSets.push_back({
+        tmpDir->GetPathName(),
+        "",
+        {
+            reportFileName.GetFileName().ToString()
+        }
+    });
+    shared_ptr<Session> session = MIKTEX_SESSION();
+    auto userLogDirectory = session->GetSpecialPath(SpecialPath::UserLogDirectory);
+    if (Directory::Exists(userLogDirectory))
+    {
+        fileSets.push_back({
+            userLogDirectory,
+            "logs/user/",
+            {
+                "."
+            }
+        });
+    }
+    auto commonLogDirectory = session->GetSpecialPath(SpecialPath::CommonLogDirectory);
+    if (userLogDirectory != commonLogDirectory && Directory::Exists(commonLogDirectory))
+    {
+        fileSets.push_back({
+            commonLogDirectory,
+            "logs/common/",
+            {
+                "."
+            }
+        });
+    }
+    auto creator = Creator::New(ArchiveFileType::TarBzip2);
+    creator->Create(PathName(outputFileName), fileSets);
+}
