@@ -437,16 +437,10 @@ EXTERN unsigned int debug INIT(0);
 
 /************************timing stuff*********************/
 #ifdef TIMING
-# ifdef TIME_WITH_SYS_TIME
-#  include <sys/time.h>
-#  include <time.h>
-# else
 #  ifdef HAVE_SYS_TIME_H
-#   include <sys/time.h>
-#  else
-#   include <time.h>
+#    include <sys/time.h>
 #  endif
-# endif
+#  include <time.h>
 EXTERN double timer INIT(0);
 EXTERN double my_tic,my_toc INIT(0);
 EXTERN int      ndone INIT(0);          /* number of pages converted       */
