@@ -1,9 +1,24 @@
+/**
+ * Copyright (C) 2017-2020  Stefan Löffler
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the Free
+ * Software Foundation; either version 2, or (at your option) any later
+ * version.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
+ */
 #ifndef __PAPER_SIZES_H
 #define __PAPER_SIZES_H
 
-#include <QString>
-#include <QSizeF>
 #include <QCoreApplication>
+#include <QSizeF>
+#include <QString>
+
+namespace QtPDF {
 
 class PaperSize
 {
@@ -32,4 +47,6 @@ protected:
   LengthUnit _lengthUnit;
 };
 
+} // namespace QtPDF
+
 #endif // !defined(__PAPER_SIZES_H)
