@@ -12,12 +12,11 @@
 
 namespace camp {
 
-#ifdef HAVE_GL
+#ifdef HAVE_LIBGLM
 
 struct BezierPatch
 {
   vertexBuffer data;
-
   bool transparent;
   bool color;
   double epsilon;
@@ -29,8 +28,16 @@ struct BezierPatch
   vertexFunction pvertex;
   bool Onscreen;
 
+  BezierPatch() : transparent(false), color(false), Onscreen(true) {}
+
   void init(double res);
 
+  void init(double res, GLfloat *colors) {
+    transparent=false;
+    color=colors;
+    init(res);
+  }
+
   triple normal(triple left3, triple left2, triple left1, triple middle,
                 triple right1, triple right2, triple right3) {
     triple lp=3.0*(left1-middle);
@@ -155,11 +162,11 @@ struct BezierPatch
   void queue(const triple *g, bool straight, double ratio, bool Transparent,
              GLfloat *colors=NULL) {
     data.clear();
-    notRendered();
     Onscreen=true;
     transparent=Transparent;
     color=colors;
-    init(pixel*ratio);
+    notRendered();
+    init(pixelResolution*ratio);
     render(g,straight,colors);
   }
 
