@@ -62,6 +62,13 @@ public:
   virtual types::ty *trans(coenv &e, bool tacit = false) = 0;
 
   virtual trans::tyEntry *transAsTyEntry(coenv &e, record *where);
+
+  virtual operator string() const = 0;
+#ifdef USEGC
+  operator std::string() const {
+    return mem::stdString(this->operator string());
+  }
+#endif
 };
 
 class nameTy : public ty {
@@ -74,10 +81,12 @@ public:
   nameTy(name *id)
     : ty(id->getPos()), id(id) {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
-  types::ty *trans(coenv &e, bool tacit = false);
-  trans::tyEntry *transAsTyEntry(coenv &e, record *where);
+  types::ty *trans(coenv &e, bool tacit = false) override;
+  trans::tyEntry *transAsTyEntry(coenv &e, record *where) override;
+
+  virtual operator string() const override;
 };
 
 class dimensions : public absyn {
@@ -95,7 +104,7 @@ public:
     return depth;
   }
 
-  types::array *truetype(types::ty *base);
+  types::array *truetype(types::ty *base, bool tacit=false);
 };
 
 class arrayTy : public ty {
@@ -109,15 +118,17 @@ public:
   arrayTy(name *id, dimensions *dims)
     : ty(dims->getPos()), cell(new nameTy(id)), dims(dims) {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
-  void addOps(coenv &e, record *r);
+  void addOps(coenv &e, record *r) override;
 
-  types::ty *trans(coenv &e, bool tacit = false);
+  types::ty *trans(coenv &e, bool tacit = false) override;
+
+  operator string() const override;
 };
 
-// Similar to varEntryExp, this helper class always translates to the same fixed
-// type.
+// Similar to varEntryExp, this helper class always translates to the same
+// fixed type.
 class tyEntryTy : public ty {
   trans::tyEntry *ent;
 public:
@@ -126,12 +137,13 @@ public:
 
   tyEntryTy(position pos, types::ty *t);
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
-  types::ty *trans(coenv &e, bool tacit = false);
-  trans::tyEntry *transAsTyEntry(coenv &, record *) {
+  types::ty *trans(coenv &e, bool tacit = false) override;
+  trans::tyEntry *transAsTyEntry(coenv &, record *) override {
     return ent;
   }
+  operator string() const override;
 };
 
 // Runnable is anything that can be executed by the program, including
@@ -187,6 +199,19 @@ public:
   { return false; }
 };
 
+// Forward declaration.
+class formals;
+
+class namedTyEntry : public gc {
+public:
+  symbol dest;
+  trans::tyEntry *ent;
+  position pos;
+  namedTyEntry(position pos, symbol dest, trans::tyEntry *ent)
+    : dest(dest), ent(ent), pos(pos) {}
+};
+
+
 class block : public runnable {
 public:
   mem::list<runnable *> stms;
@@ -208,16 +233,27 @@ public:
     stms.push_back(r);
   }
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
-  void trans(coenv &e);
+  void trans(coenv &e) override;
 
-  void transAsField(coenv &e, record *r);
+  void transAsField(coenv &e, record *r) override;
+
+  bool transAsTemplatedField(
+      coenv &e, record *r, mem::vector<absyntax::namedTyEntry*>* args
+  );
 
   void transAsRecordBody(coenv &e, record *r);
+  bool transAsTemplatedRecordBody(
+      coenv &e, record *r, mem::vector<absyntax::namedTyEntry*> *args
+  );
 
   types::record *transAsFile(genv& ge, symbol id);
 
+  types::record *transAsTemplatedFile(
+      genv& ge, symbol id, mem::vector<absyntax::namedTyEntry*> *args
+  );
+
   // If the block can be interpreted as a single vardec, return that vardec
   // (otherwise 0).
   vardec *asVardec();
@@ -233,7 +269,9 @@ public:
   // }
   //
   // is not guaranteed to return.
-  bool returns();
+  bool returns() override;
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
 class modifierList : public absyn {
@@ -307,7 +345,7 @@ public:
   decidstart(position pos, symbol id, dimensions *dims = 0)
     : absyn(pos), id(id), dims(dims) {}
 
-  virtual void prettyprint(ostream &out, Int indent);
+  virtual void prettyprint(ostream &out, Int indent) override;
 
   virtual types::ty *getType(types::ty *base, coenv &, bool = false);
   virtual trans::tyEntry *getTyEntry(trans::tyEntry *base, coenv &e,
@@ -319,10 +357,12 @@ public:
 
   virtual symbol getName()
   { return id; }
-};
 
-// Forward declaration.
-class formals;
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
+  void createSymMapWType(
+      AsymptoteLsp::SymbolContext* symContext, absyntax::ty* base
+  );
+};
 
 class fundecidstart : public decidstart {
   formals *params;
@@ -352,7 +392,7 @@ public:
   decid(position pos, decidstart *start, varinit *init = 0)
     : absyn(pos), start(start), init(init) {}
 
-  virtual void prettyprint(ostream &out, Int indent);
+  virtual void prettyprint(ostream &out, Int indent) override;
 
   virtual void transAsField(coenv &e, record *r, types::ty *base);
 
@@ -360,6 +400,11 @@ public:
   virtual void transAsTypedefField(coenv &e, trans::tyEntry *base, record *r);
 
   decidstart *getStart() { return start; }
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
+  void createSymMapWType(
+      AsymptoteLsp::SymbolContext* symContext, absyntax::ty* base
+  );
 };
 
 class decidlist : public absyn {
@@ -375,7 +420,7 @@ public:
     decs.push_back(p);
   }
 
-  virtual void prettyprint(ostream &out, Int indent);
+  virtual void prettyprint(ostream &out, Int indent) override;
 
   virtual void transAsField(coenv &e, record *r, types::ty *base);
 
@@ -390,6 +435,11 @@ public:
     else
       return 0;
   }
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
+  void createSymMapWType(
+      AsymptoteLsp::SymbolContext* symContext, absyntax::ty* base
+  );
 };
 
 class dec : public runnable {
@@ -397,10 +447,10 @@ public:
   dec(position pos)
     : runnable(pos) {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
   // Declarations can be public or private.
-  bool allowPermissions()
+  bool allowPermissions() override
   { return true; }
 };
 
@@ -420,9 +470,9 @@ public:
   {
     decs->add(di);
   }
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
-  void transAsField(coenv &e, record *r)
+  void transAsField(coenv &e, record *r) override
   {
     base->addOps(e, r);
     decs->transAsField(e, r, base->trans(e));
@@ -438,6 +488,8 @@ public:
   // If the vardec encodes a single declaration, return the type of that
   // declaration (otherwise 0).
   types::ty *singleGetType(coenv& e);
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
 struct idpair : public absyn {
@@ -462,7 +514,7 @@ struct idpair : public absyn {
     }
   }
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
   // Translates as: access src as dest;
   void transAsAccess(coenv &e, record *r);
@@ -471,6 +523,8 @@ struct idpair : public absyn {
   // where _ is the qualifier record with source as its fields and types.
   void transAsUnravel(coenv &e, record *r,
                       protoenv &source, varEntry *qualifier);
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
 struct idpairlist : public gc {
@@ -486,6 +540,17 @@ struct idpairlist : public gc {
 
   void transAsUnravel(coenv &e, record *r,
                       protoenv &source, varEntry *qualifier);
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext);
+
+  template<typename TFn>
+  void processListFn(TFn const& fn)
+  {
+    for (auto* idp : base)
+    {
+      fn(idp->src, idp->dest);
+    }
+  }
 };
 
 extern idpairlist * const WILDCARD;
@@ -498,13 +563,94 @@ public:
   accessdec(position pos, idpairlist *base)
     : dec(pos), base(base) {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
-  void transAsField(coenv &e, record *r) {
+  void transAsField(coenv &e, record *r) override {
     base->transAsAccess(e,r);
   }
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
+class badDec : public dec {
+  position errorPos;
+  string errorMessage;
+
+public:
+  badDec(position pos, position errorPos, string errorMessage)
+      : dec(pos), errorPos(errorPos), errorMessage(errorMessage) {}
+
+  void transAsField(coenv&, record*) override {
+    em.error(errorPos);
+    em << errorMessage;
+  }
+};
+
+// Accesses the file with specified types added to the type environment.
+class templateAccessDec : public dec {
+  symbol src; // The name of the module to access.
+  formals *args;
+  symbol dest;  // What to call it in the local environment.
+  bool valid;
+  position expectedAsPos;
+
+public:
+  templateAccessDec(position pos, symbol src, formals* args, symbol as,
+                    symbol dest, position asPos)
+      : dec(pos), src(src), args(args), dest(dest),
+        valid(as == symbol::trans("as")), expectedAsPos(asPos) {}
+
+  bool checkValidity() {
+    if (!valid) {
+      em.error(expectedAsPos);
+      em << "expected 'as'";
+      return false;
+    }
+    return true;
+  }
+
+  void transAsField(coenv& e, record* r) override;
+};
+
+class typeParam : public absyn {
+  const symbol paramSym;
+public:
+  typeParam(position pos, symbol paramSym)
+    : absyn(pos), paramSym(paramSym) {}
+
+  bool transAsParamMatcher(coenv &e, namedTyEntry *arg);
+
+  void prettyprint(ostream &out, Int indent);
+};
+
+class typeParamList : public absyn {
+  mem::vector<typeParam*> params;
+
+public:
+  typeParamList(position pos) : absyn(pos) {}
+
+  void add(typeParam *tp);
+
+  bool transAsParamMatcher(coenv &e, mem::vector<namedTyEntry*> *args);
+
+  void prettyprint(ostream &out, Int indent);
+};
+
+
+class receiveTypedefDec : public dec {
+  typeParamList* params;
+
+public:
+  receiveTypedefDec(position pos, typeParamList* params) 
+    : dec(pos), params(params) {}
+
+  void transAsField(coenv& e, record *r) override;
+  bool transAsParamMatcher(
+      coenv& e, record *r, mem::vector<namedTyEntry*> *args
+  );
+};
+
+
 // Abstract base class for
 //   from _ access _;  (fromaccessdec)
 // and
@@ -513,8 +659,8 @@ class fromdec : public dec {
 protected:
   struct qualifier {
     // The varEntry holds the location and the type of the highest framed
-    // structure that can be put on the stack.  The record holds the actual type
-    // of the qualifier.
+    // structure that can be put on the stack.  The record holds the actual
+    // type of the qualifier.
     // For example:
     //   struct A {
     //     struct B {
@@ -542,9 +688,9 @@ public:
   fromdec(position pos, idpairlist *fields)
     : dec(pos), fields(fields) {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
-  void transAsField(coenv &e, record *r);
+  void transAsField(coenv &e, record *r) override;
 };
 
 // An unravel declaration dumps fields and types of a record into the local
@@ -552,25 +698,31 @@ public:
 class unraveldec : public fromdec {
   name *id;
 
-  qualifier getQualifier(coenv &e, record *);
+  qualifier getQualifier(coenv &e, record *) override;
 public:
   unraveldec(position pos, name *id, idpairlist *fields)
     : fromdec(pos, fields), id(id) {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
 // A fromaccess declaration dumps fields and types of a module into the local
 // scope.  It does not add the module as a variable in the local scope.
 class fromaccessdec : public fromdec {
   symbol id;
+  formals *templateArgs;
 
-  qualifier getQualifier(coenv &e, record *r);
+  qualifier getQualifier(coenv &e, record *r) override;
 public:
-  fromaccessdec(position pos, symbol id, idpairlist *fields)
-    : fromdec(pos, fields), id(id) {}
+  fromaccessdec(
+      position pos, symbol id, idpairlist *fields, formals *templateArgs = 0
+  ) : fromdec(pos, fields), id(id), templateArgs(templateArgs) {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
 // An import declaration dumps fields and types of a module into the local
@@ -587,15 +739,17 @@ public:
     base.add(new unraveldec(pos, new simpleName(pos, id->dest), WILDCARD));
   }
 
-  void trans(coenv &e) {
+  void trans(coenv &e) override {
     base.trans(e);
   }
 
-  void transAsField(coenv &e, record *r) {
+  void transAsField(coenv &e, record *r) override {
     base.transAsField(e, r);
   }
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
 // Parses the file given, and translates the resulting runnables as if they
@@ -609,10 +763,12 @@ public:
   includedec(position pos, symbol id)
     : dec(pos), filename(id) {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
   void loadFailed(coenv &e);
 
-  void transAsField(coenv &e, record *r);
+  void transAsField(coenv &e, record *r) override;
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
 // Types defined from others in typedef.
@@ -646,9 +802,11 @@ public:
   virtual ~recorddec()
   {}
 
-  void prettyprint(ostream &out, Int indent);
+  void prettyprint(ostream &out, Int indent) override;
 
-  void transAsField(coenv &e, record *parent);
+  void transAsField(coenv &e, record *parent) override;
+
+  void createSymMap(AsymptoteLsp::SymbolContext* symContext) override;
 };
 
 // Returns a runnable that facilitates the autoplain feature.
@@ -659,4 +817,3 @@ void addVar(coenv &e, record *r, varEntry *v, symbol id);
 } // namespace absyntax
 
 #endif
-
